@@ -273,9 +273,11 @@ main(int argc, char *const *argv)
         return 1;
     }
 
-    /* ngx_crc32_init() requires ngx_cacheline_size set in ngx_os_init() */
+    /*
+     * ngx_crc32_table_init() requires ngx_cacheline_size set in ngx_os_init()
+     */
 
-    if (ngx_crc32_init() != NGX_OK) {
+    if (ngx_crc32_table_init() != NGX_OK) {
         return 1;
     }
 
@@ -656,10 +658,6 @@ ngx_getopt(ngx_cycle_t *cycle, int argc, char *const *argv)
         cycle->conf_file.data = (u_char *) NGX_CONF_PATH;
     }
 
-    if (ngx_conf_full_name(cycle, &cycle->conf_file) == NGX_ERROR) {
-        return NGX_ERROR;
-    }
-
     return NGX_OK;
 }
 
@@ -760,12 +758,6 @@ ngx_core_module_init_conf(ngx_cycle_t *cycle, void *conf)
 {
     ngx_core_conf_t  *ccf = conf;
 
-#if !(NGX_WIN32)
-    ngx_str_t       lock_file;
-    struct group   *grp;
-    struct passwd  *pwd;
-#endif
-
     ngx_conf_init_value(ccf->daemon, 1);
     ngx_conf_init_value(ccf->master, 1);
     ngx_conf_init_msec_value(ccf->timer_resolution, 0);
@@ -798,6 +790,8 @@ ngx_core_module_init_conf(ngx_cycle_t *cycle, void *conf)
 #if !(NGX_WIN32)
 
     if (ccf->user == (uid_t) NGX_CONF_UNSET_UINT && geteuid() == 0) {
+        struct group   *grp;
+        struct passwd  *pwd;
 
         ngx_set_errno(0);
         pwd = getpwnam(NGX_USER);
@@ -826,7 +820,7 @@ ngx_core_module_init_conf(ngx_cycle_t *cycle, void *conf)
         ccf->pid.data = (u_char *) NGX_PID_PATH;
     }
 
-    if (ngx_conf_full_name(cycle, &ccf->pid) == NGX_ERROR) {
+    if (ngx_conf_full_name(cycle, &ccf->pid, 0) == NGX_ERROR) {
         return NGX_CONF_ERROR;
     }
 
@@ -846,10 +840,13 @@ ngx_core_module_init_conf(ngx_cycle_t *cycle, void *conf)
         ccf->lock_file.data = (u_char *) NGX_LOCK_PATH;
     }
 
-    if (ngx_conf_full_name(cycle, &ccf->lock_file) == NGX_ERROR) {
+    if (ngx_conf_full_name(cycle, &ccf->lock_file, 0) == NGX_ERROR) {
         return NGX_CONF_ERROR;
     }
 
+    {
+    ngx_str_t  lock_file;
+
     lock_file = cycle->old_cycle->lock_file;
 
     if (lock_file.len) {
@@ -883,6 +880,7 @@ ngx_core_module_init_conf(ngx_cycle_t *cycle, void *conf)
                               ccf->lock_file.len),
                    ".accept", sizeof(".accept"));
     }
+    }
 
 #endif
 
