@@ -335,7 +335,7 @@ ngx_http_special_response_handler(ngx_http_request_t *r, ngx_int_t error)
     ngx_log_debug2(NGX_LOG_DEBUG_HTTP, r->connection->log, 0,
                    "http special response: %d, \"%V\"", error, &r->uri);
 
-    rc = ngx_http_discard_body(r);
+    rc = ngx_http_discard_request_body(r);
 
     if (rc == NGX_HTTP_INTERNAL_SERVER_ERROR) {
         error = NGX_HTTP_INTERNAL_SERVER_ERROR;
@@ -439,13 +439,8 @@ ngx_http_send_error_page(ngx_http_request_t *r, ngx_http_err_page_t *err_page)
 
     r->err_status = err_page->overwrite;
 
-    r->method = NGX_HTTP_GET;
-    r->method_name = ngx_http_get_name;
-
     r->zero_in_uri = 0;
 
-    args = NULL;
-
     if (err_page->uri_lengths) {
         if (ngx_http_script_run(r, &u, err_page->uri_lengths->elts, 0,
                                 err_page->uri_values->elts)
@@ -456,6 +451,7 @@ ngx_http_send_error_page(ngx_http_request_t *r, ngx_http_err_page_t *err_page)
 
         p = u.data;
         uri = &u;
+        args = NULL;
 
         if (*p == '/') {
 
@@ -491,9 +487,14 @@ ngx_http_send_error_page(ngx_http_request_t *r, ngx_http_err_page_t *err_page)
 
     } else {
         uri = &err_page->uri;
+        args = &err_page->args;
     }
 
     if (uri->data[0] == '/') {
+
+        r->method = NGX_HTTP_GET;
+        r->method_name = ngx_http_get_name;
+
         return ngx_http_internal_redirect(r, uri, args);
     }
 
